/***************************************************************************
 *  Copyright (C) 2003 Daniel Franke                                       *
 *                                                                         *
 *  Authors:                                                               *
 *   	Daniel Franke <daniel.franke@imbs.uni-luebeck.de>                  *
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 ***************************************************************************/

#include <algorithm>
#include <functional> 
using std::for_each;
using std::unary_function;

#include "db.h"

struct db db;

template <typename T> struct appendTo : public unary_function<typename T::value_type, void> {
        appendTo(T& t) : tableref(t) { }

        void operator()(const typename T::value_type& it) const {
                tableref.append(it);
        }

        T& tableref;
};

PersonTable PersonTable::selectWherePID(const IdType& pid) {
	PersonTable subtable;

	for (iterator it = begin(); it != end(); ++it) 
		if (((*it).second)->prsn_pid == pid)
			subtable.append(*it);

	return subtable;
}



FrequencyTable FrequencyTable::selectWhereLocusID(const IdType& locid) {
	FrequencyTable subtable;

	std::for_each(mTable.lower_bound(locid),
		      mTable.upper_bound(locid),
		      appendTo<FrequencyTable>(subtable));

	return subtable;
}

GenotypeTable GenotypeTable::selectWherePID(const IdType& pid) {
	GenotypeTable subtable;

	std::for_each(mTable.lower_bound(pid), 
		      mTable.upper_bound(pid), 
		      appendTo<GenotypeTable>(subtable));

	return subtable;
}


GenotypeTable GenotypeTable::selectWhereLocusID(const IdType& locid) {
	GenotypeTable subtable;

	for (iterator it = begin(); it != end(); ++it) 
		if (((*it).second)->geno_locid == locid)
			subtable.append((*it).first, (*it).second);

	return subtable;
}

void GenotypeTable::clear(void) {
	for (iterator it = begin(); it != end(); ++it)
		delete (*it).second;

	table<const IdType, struct Genotype *, IdType>::clear();
}


LocusTable LocusTable::selectWhereID(const IdType& id) {
	LocusTable subtable;

	for (iterator it = begin(); it != end(); ++it)
		if (((*it).second)->loc_id == id)
			subtable.append((*it).first, (*it).second);

	return subtable;
}


PhenotypeTable PhenotypeTable::selectWherePID(const IdType& pid) {
	PhenotypeTable subtable;

	for_each(mTable.lower_bound(pid), 
		 mTable.upper_bound(pid),
		 appendTo<PhenotypeTable>(subtable));

	return subtable;
}

void PhenotypeTable::clear(void) {
	for (iterator it = begin(); it != end(); ++it)
		delete (*it).second;

	table<const IdType, struct Phenotype *, IdType>::clear();
}

